# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright (C) 2013 Canonical Ltd.
# Author: Sergio Schvezov <sergio.schvezov@canonical.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import configobj
import contextlib
import fcntl
import hashlib
import logging
import os
import requests
import subprocess

from xdg.BaseDirectory import xdg_config_home


log = logging.getLogger()


@contextlib.contextmanager
def flocked(lockfile):
    lockfile += '.lock'
    with open(lockfile, 'w') as f:
        log.debug('Aquiring lock for %s', lockfile)
        try:
            fcntl.lockf(f, fcntl.LOCK_EX)
            yield
        finally:
            log.debug('Releasing lock for %s', lockfile)
            fcntl.lockf(f, fcntl.LOCK_UN)


def setup_download_directory(download_dir):
    '''
    Tries to create the download directory from XDG_DOWNLOAD_DIR or sets
    an alternative one.

    Returns path to directory
    '''
    log.info('Download directory set to %s' % download_dir)
    if not os.path.exists(download_dir):
        log.info('Creating %s' % download_dir)
        try:
            os.makedirs(download_dir)
        except OSError as e:
            if e.errno == 17:
                pass
            else:
                raise e


def get_full_path(subdir):
    try:
        userdirs_file = os.path.join(xdg_config_home, 'user-dirs.dirs')
        userdirs_config = configobj.ConfigObj(userdirs_file, encoding='utf-8')
        userdirs_download = os.path.expandvars(
            userdirs_config['XDG_DOWNLOAD_DIR'])
        download_dir = userdirs_download
    except KeyError:
        download_dir = os.path.expandvars('$HOME')
        log.warning('XDG_DOWNLOAD_DIR could not be read')
    directory = os.path.join(download_dir, subdir)
    setup_download_directory(directory)
    return directory


def get_sha256_hash(string):
    return hashlib.sha256(string).hexdigest()


def checksum_verify(file_path, file_hash, sum_method=hashlib.sha256):
    '''Returns the checksum for a file with a specified algorightm.'''
    file_sum = sum_method()
    log.debug('Verifying file: %s against: %s' % (file_path, file_hash))
    if not os.path.exists(file_path):
        log.debug('File %s not found' % file_path)
        return False
    with open(file_path, 'rb') as f:
        for file_chunk in iter(
                lambda: f.read(file_sum.block_size * 128), b''):
            file_sum.update(file_chunk)
    if file_hash == file_sum.hexdigest():
        return True
    else:
        log.debug('Calculated sum mismatch calculated %s != %s' %
                  (file_sum.hexdigest(), file_hash))
    return False


def _download(uri, path):
    if uri.startswith('http://10.97.0.26'):
        subprocess.check_call(['curl',
                               '-L',
                               '-C',
                               '-',
                               uri,
                               '-o',
                               path])
    else:
        subprocess.check_call(['wget',
                               '-c',
                               uri,
                               '-O',
                               path])


def download_sig(artifact):
    '''Downloads an artifact into target.'''
    log.info('Downloading %s to %s' % (artifact.sig_uri, artifact.sig_path))
    with flocked(artifact._sig_path):
        _download(artifact.sig_uri, artifact.sig_path)


def download(artifact):
    '''Downloads an artifact into target.'''
    log.info('Downloading %s to %s' % (artifact.uri, artifact.path))
    with flocked(artifact._path):
        _download(artifact.uri, artifact.path)


def get_content(uri):
    '''Fetches the SHA256 sum file from cdimage.'''
    content_request = requests.get(uri)
    if content_request.status_code != 200:
        return None
    else:
        return content_request.content


def get_ubuntu_stamp(uri):
    '''Downloads the jenkins build id from stamp file'''
    try:
        ubuntu_stamp = requests.get('%s/quantal-ubuntu_stamp' % uri)
        if ubuntu_stamp.status_code != 200:
            ubuntu_stamp = requests.get('%s/ubuntu_stamp' % uri)
        if ubuntu_stamp.status_code != 200:
            log.error('Latest build detection not supported... bailing')
            exit(1)
        # Make list and get rid of empties
        build_data = filter(lambda x: x.startswith('JENKINS_BUILD='),
                            ubuntu_stamp.content.split('\n'))
        jenkins_build_id = build_data[0].split('=')[1]
    except (requests.HTTPError, requests.Timeout, requests.ConnectionError):
        log.error('Could not download build data from jenkins... bailing')
        exit(1)
    except IndexError:
        raise EnvironmentError('Jenkins data format has changed, incompatible')
    return jenkins_build_id



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright (C) 2013 Canonical Ltd.
# Author: Sergio Schvezov <sergio.schvezov@canonical.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import os.path

from phabletutils import downloads
from phabletutils import fileutils
from phabletutils import resources


def _get_json(device, channel, uri):
    json_index_uri = '%s/%s/%s/index.json' % \
        (uri, channel, device)
    json_content = downloads.get_content(json_index_uri)
    if not json_content:
        raise RuntimeError('%s cannot be retrieved' % json_index_uri)
    return json.loads(json_content)


def get_json_from_index(device, channel, index, uri):
    """Returns json index for device"""
    index -= 1
    json_index = _get_json(device, channel, uri)
    json_dict = sorted([entry for entry in json_index['images']
                       if entry['type'] == "full"],
                       key=lambda entry: entry['version'])[index]
    return json_dict


def get_json_from_version(device, channel, version, uri):
    for entry in _get_json(device, channel, uri)['images']:
        if entry['type'] == 'full' and entry['version'] == version:
            return entry
    raise RuntimeError('Version %s cannot be retrieved' % version)


def get_files(download_dir, uri, json):
    files = {}
    command_part = ''
    files = []
    for entry in sorted(json['files'], key=lambda entry: entry['order']):
        filename = entry['path'].split("/")[-1]
        signame = entry['signature'].split("/")[-1]
        paths = {}
        for path in ('path', 'signature'):
            paths[path] = fileutils.create_path(
                download_dir, os.path.dirname(entry[path].strip(os.path.sep)))
        f = resources.SignedFile(
            file_path=os.path.join(paths['path'], filename),
            sig_path=os.path.join(paths['signature'], signame),
            file_uri='%s%s' % (uri, entry['path']),
            sig_uri='%s%s' % (uri, entry['signature']),
            file_hash=entry['checksum'])
        files.append(f)
        command_part += 'update %s %s\n' % (filename, signame)
    gpg_download_dir = fileutils.create_temp_dir()
    for keyring in ('image-master', 'image-signing'):
        filename = '%s.tar.xz' % keyring
        signame = '%s.asc' % filename
        f = resources.SignedFile(
            file_path=os.path.join(gpg_download_dir, filename),
            sig_path=os.path.join(gpg_download_dir, signame),
            file_uri='%s/gpg/%s' % (uri, filename),
            sig_uri='%s/gpg/%s.asc' % (uri, filename),
            file_hash=None)
        files.append(f)
    return files, command_part



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright (C) 2013 Canonical Ltd.
# Author: Sergio Schvezov <sergio.schvezov@canonical.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import subprocess
import logging

from time import sleep

log = logging.getLogger()


def call(args):
    subprocess.check_call(args, shell=True)


def check_output(args):
    return subprocess.check_output(args, shell=True)


class Device(object):
    '''Android device.'''

    def __init__(self, cmd, device=None):
        if device:
            log.info('Setting device %s' % device)
            self._cmd = '%s -s %s %%s' % (cmd, device)
        else:
            self._cmd = '%s %%s' % cmd
        self._device = device

    @property
    def device(self):
        '''Returns the device used for the adb interface.'''
        return self._device


class AndroidBridge(Device):
    '''Interface to adb.'''

    def __init__(self, device=None):
        '''Initializes an adb interface attached to a given device.'''
        super(AndroidBridge, self).__init__(device=device, cmd='adb')

    def start(self):
        '''Attempts to start adb if not running.'''
        log.debug('Starting adb server')
        cmd = 'start-server'
        call(self._cmd % cmd)

    def push(self, src, dst):
        '''Performs and adb push.'''
        log.info('Pushing %s to %s' % (src, dst))
        cmd = 'push %s %s' % (src, dst)
        call(self._cmd % cmd)

    def pull(self, src, dst):
        '''Performs and adb pull.'''
        log.info('Pulling %s to %s' % (src, dst))
        cmd = 'pull %s %s' % (src, dst)
        call(self._cmd % cmd)

    def wait_for_device(self, wait=2):
        '''Waits for device.'''
        # Hack wait to avoid LP:1176929
        log.debug('Waiting for device...')
        sleep(wait)
        call(self._cmd % 'wait-for-device')
        log.debug('Device is available')

    def wait_for_network(self):
        '''Wait for networking to become active on the device.'''
        cmd = 'phablet-network --skip-setup'
        if self._device:
            cmd += ' -s %s' % self._device
        log.info('Waiting for networking to become active...')
        call(cmd)
        log.info('Network is active')

    def wait_for_recovery(self):
        '''Waits for recovery.'''
        # adb wait-for-device when in recovery stalls, this
        # works around that issue.
        log.info('Waiting for recovery image to boot')
        # Initial wait time
        wait_try = 200
        reboot_success = False
        for wait in range(wait_try):
            sleep(10)
            try:
                output = self.shell('ls /sbin/recovery 2>/dev/null')
            except:
                output = None
            if output:
                reboot_success = True
                break
            wait += 1
        if not reboot_success:
            raise RuntimeError('Wait for recovery expired')
        log.info('Wait for recovery image to boot complete')

    def root(self):
        '''Set device to work as root.'''
        call(self._cmd % 'root')
        self.wait_for_device()

    def chmod(self, filename, mode):
        '''Performs a chmod on target device.'''
        cmd = 'shell chmod %s %s' % (mode, filename)
        call(self._cmd % cmd)

    def chown(self, user, path):
        '''Performs a chmod on target device.'''
        cmd = 'shell chown -R %s:%s %s' % (user, user, path)
        call(self._cmd % cmd)

    def getprop(self, android_property):
        '''Returns an android property.'''
        cmd = 'shell getprop %s ' % (android_property)
        return check_output(self._cmd % cmd)

    def tcp_forward(self, src, dst):
        '''Creates a tcp forwarding rule.'''
        cmd = 'forward tcp:%s tcp:%s' % (src, dst)
        call(self._cmd % cmd)

    def shell(self, command, ignore_errors=True):
        '''Runs shell command and returns output'''
        if not ignore_errors:
            command = '"%s ; echo ADB_RC=\\$?"' % command
        cmd = 'shell %s' % command
        output = check_output(self._cmd % cmd)
        if not ignore_errors:
            # get the last line of output
            rc = output.split('\r\n')[-2]
            if not rc.startswith('ADB_RC='):
                raise RuntimeError('unexpected output: %s' % rc)
            rc = int(rc[7:])
            if rc != 0:
                raise subprocess.CalledProcessError(rc, command, output)
        return output

    def chroot(self, command, root='data/ubuntu'):
        '''Runs command in chroot.'''
        log.debug('Running in chroot: %s' % command)
        cmd = 'shell "PATH=/usr/sbin:/usr/bin:/sbin:/bin ' \
              'system/xbin/chroot %s %s"' % (root, command)
        call(self._cmd % cmd)

    def reboot(self, recovery=False, bootloader=False):
        '''Reboots device.'''
        log.info('Restarting device... wait')
        if recovery:
            cmd = 'reboot recovery'
        elif bootloader:
            cmd = 'reboot bootloader'
        else:
            cmd = 'reboot'
        call(self._cmd % cmd)
        if recovery:
            sleep(20)
        log.info('Restarting device... wait complete')


class Fastboot(Device):
    '''Interface to fastboot'''

    def __init__(self, device=None):
        super(Fastboot, self).__init__(device=device, cmd='fastboot')

    def flash(self, image_partition, image_file):
        log.info('Flashing %s to %s' % (image_partition, image_file))
        cmd = 'flash %s %s' % (image_partition, image_file)
        call(self._cmd % cmd)

    def flash_system(self, image_file):
        self.flash('system', image_file)

    def flash_recovery(self, image_file):
        self.flash('recovery', image_file)

    def flash_boot(self, image_file):
        self.flash('boot', image_file)

    def reboot(self):
        log.info('Rebooting device')
        cmd = 'reboot'
        call(self._cmd % cmd)

    def boot(self, image_file=None):
        if image_file:
            log.info('Booting %s' % image_file)
            cmd = 'boot %s' % image_file
        else:
            log.info('Booting OS')
            cmd = 'boot'
        call(self._cmd % cmd)

    def wipe(self):
        log.info('Wiping all userdata')
        cmd = '-w'
        call(self._cmd % cmd)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright (C) 2013 Canonical Ltd.
# Author: Sergio Schvezov <sergio.schvezov@canonical.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import logging
import tempfile
import urllib
import urlparse

from phabletutils import cdimage
from phabletutils import environment
from phabletutils import resources
from phabletutils import settings

log = logging.getLogger()


class PathAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        log.debug('PathAction: %r %r %r' %
                  (namespace, values, option_string))
        uri = urlparse.urlparse(values)
        if not uri.scheme or uri.scheme == 'file':
            if '%' in uri.path:
                zip_file_path = urllib.unquote(
                    urlparse.urlparse(uri.path))
            else:
                zip_file_path = uri.path
            zip_file_uri = None
            check = False
        elif uri.scheme == 'http' or uri.scheme == 'https':
            zip_file_path = tempfile.mktemp()
            zip_file_uri = values
            check = True
        log.debug('Download from %s, path on disk %s' %
                  (zip_file_uri, zip_file_path))
        artifact = resources.File(file_path=zip_file_path,
                                  file_uri=zip_file_uri,
                                  check=check)
        setattr(namespace, self.dest, artifact)


class RevisionListAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        log.debug('RevisionListAction: %r %r %r' %
                  (namespace, values, option_string))
        project = 'ubuntu-touch-preview'
        uri = '%s/%s' % (settings.cdimage_uri_base, project)
        setattr(namespace, 'func', environment.list_revisions)
        setattr(namespace, 'uri', uri)


class DeprecatedBackup(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        log.warn('--no-backup is deprecated, use --bootstrap instead')
        setattr(namespace, 'bootstrap', True)


class RevisionAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        log.debug('RevisionAction: %r %r %r' %
                  (namespace, values, option_string))
        project = 'ubuntu-touch-preview'
        base_uri = '%s/%s' % (settings.cdimage_uri_base, project)
        if values:
            revision_split = values.split('/')
            if len(revision_split) != 2:
                raise EnvironmentError(
                    'Improper use of revision, needs to be formatted like'
                    '[series]/[revision]. Use --list-revisions to find'
                    'the available revisions on cdimage')
            series = revision_split[0]
            build = revision_split[1]
        else:
            series, build = cdimage.get_latest_revision(base_uri)
        uri = '%s/%s/%s' % (base_uri, series, build)
        setattr(namespace, self.dest, True)
        setattr(namespace, 'series', series)
        setattr(namespace, 'build', build)
        setattr(namespace, 'uri', uri)


def cdimage_touch(parent_parser, parents):
    parser = parent_parser.add_parser(
        'cdimage-touch', parents=parents,
        help='Provisions the device with a CDimage build of Ubuntu Touch.')
    parser.set_defaults(func=environment.setup_cdimage_touch,
                        project='ubuntu-touch',
                        series=settings.default_series,
                        build=None,
                        uri=None)
    parser.add_argument('-b',
                        '--bootstrap',
                        help='''Bootstrap the target device, this only
                                works on Nexus devices or devices that
                                use fastboot and are unlocked. All user
                                data is destroyed''',
                        action='store_true')
    group = parser.add_mutually_exclusive_group()
    group.add_argument('--pending',
                       action='store_true',
                       required=False,
                       help='Get pending link from cdimage')
    group.add_argument('-p',
                       '--base-path',
                       required=False,
                       default=None,
                       help='''Installs from base path, you must have
                               the same directory structure as if you
                               downloaded for real.
                               This option is completely offline.''')
    return parser


def ubuntu_system(parent_parser, parents):
    parser = parent_parser.add_parser(
        'ubuntu-system', parents=parents,
        help='Provisions the device with an Ubuntu Image Based Upgrade image.')
    parser.set_defaults(func=environment.setup_ubuntu_system,
                        revision=0,
                        series=settings.default_series,
                        project='imageupdates')
    parser.add_argument('--channel',
                        default='stable',
                        help='''Use an alternate system-image channel,
                                e.g. devel-proposed; defaults to stable.''')
    parser.add_argument('--revision',
                        type=int,
                        help='''Download a relative revision from current
                                (-1, -2, ...) or a specific version.''')
    parser.add_argument('--system-image-ready',
                        action='store_true',
                        help='''Explicitly state that the image already on
                                the device is a system image''')
    parser.add_argument('--alternate-server',
                        default=settings.system_image_uri,
                        help='''Use an alternate system server, the default
                                server is %(default)s''')
    group = parser.add_mutually_exclusive_group()
    group.add_argument('--no-backup',
                       action=DeprecatedBackup,
                       nargs=0,
                       help='''(Deprecated) Disables backup procedure of
                               current the Ubuntu install.''')
    group.add_argument('-b',
                       '--bootstrap',
                       action='store_true',
                       help='''Bootstraps the system into Ubuntu
                               wiping all data while doing so.''')
    return parser


def legacy(parent_parser, parents):
    parser = parent_parser.add_parser(
        'cdimage-legacy', parents=parents,
        help='Provisions the device with legacy unflipped images.')
    parser.set_defaults(func=environment.setup_cdimage_legacy,
                        project='ubuntu-touch-preview',
                        series=settings.cdimage_legacy_series,
                        build=None,
                        uri=None)
    parser.add_argument('-b',
                        '--bootstrap',
                        help='''Bootstrap the target device, this only
                                works on Nexus devices or devices that
                                use fastboot and are unlocked. All user
                                data is destroyed''',
                        action='store_true')
    group = parser.add_mutually_exclusive_group()
    group.add_argument('--list-revisions',
                       action=RevisionListAction,
                       nargs=0,
                       help='List available revisions on cdimage and exits')
    group.add_argument('-r',
                       '--revision',
                       action=RevisionAction,
                       help='''Choose a specific release to install
                               from cdimage, the format is
                               [series]/[rev].''')
    group.add_argument('-l',
                       '--latest-revision',
                       action=RevisionAction,
                       nargs=0,
                       help='''Pulls the latest tagged revision.''')
    group.add_argument('-p',
                       '--base-path',
                       required=False,
                       default=None,
                       help='''Installs from base path, you must have
                               the same directory structure as if you
                               downloaded for real.
                               This option is completely offline.''')
    return parser


def community(parent_parser, parents):
    parser = parent_parser.add_parser(
        'community', parents=parents,
        help='Provisions the device with a community supported build.')
    parser.add_argument('-d', '--device', required=True,
                        help='''Specify device to flash.
                                Find out more about flashable devices at
                                https://wiki.ubuntu.com/Touch/Devices''')
    parser.set_defaults(func=environment.setup_community,
                        series=settings.default_series)
    return parser


def common_non_system():
    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument('--device-path',
                        action=PathAction,
                        help='''uri to device zip to flash,
                                e.g.; file:///..., http://.''',)
    parser.add_argument('--ubuntu-path',
                        action=PathAction,
                        help='''uri to ubuntu zip to flash,
                                e.g.; file:///..., http://.''',)
    parser.add_argument('--wipe',
                        action='store_true',
                        help='''Wipes device data.''')
    return parser


def common_supported():
    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument('-d',
                        '--device',
                        help='''Target device to deploy.''',
                        required=False,
                        choices=settings.supported_devices)
    return parser


def common():
    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument('--debug',
                        action='store_true',
                        help='''Enable debug messages.''')
    parser.add_argument('-s',
                        '--serial',
                        help='''Device serial. Use when more than
                                one device is connected.''')
    parser.add_argument('-D',
                        '--download-only',
                        action='store_true',
                        help='Download image only, but do not flash device.')
    return parser


def get_parser():
    """
    Returns a Namespace of the parsed arguments from the command line.
    """
    parser = argparse.ArgumentParser(

        description='''phablet-flash is used to provision devices.
                       It does a best effort to deploy in different ways.''',
        epilog='''Use -h or --help after each command to learn about
                  their provisioning options.''')
    # Parsers
    common_parser = common()
    common_supported_parser = common_supported()
    common_non_system_parser = common_non_system()
    sub = parser.add_subparsers(title='Commands', metavar='')
    cdimage_touch(sub, [common_parser, common_supported_parser,
                        common_non_system_parser])
    legacy(sub, [common_parser, common_supported_parser,
                 common_non_system_parser])
    ubuntu_system(sub, [common_parser, common_supported_parser, ])
    community(sub, [common_parser, common_non_system_parser, ])
    return parser



# Copyright (C) 2013 Canonical Ltd.
# Author: Sergio Schvezov <sergio.schvezov@canonical.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Unit tests for phabletutils.environment."""

from mock import patch
from phabletutils import ubuntuimage
from testtools import TestCase
from testtools.matchers import Equals
from testtools.matchers import HasLength


@patch('phabletutils.downloads.get_content')
class TestUbuntuImage(TestCase):

    def setUp(self):
        super(TestUbuntuImage, self).setUp()
        self.device = 'mako'
        self.channel = 'daily'
        with open('tests/index.json', 'r') as f:
            self.json_content = f.read()

    def testGetLatestIndex(self, get_content_mock):
        # given
        get_content_mock.return_value = self.json_content
        # when
        json_dict = ubuntuimage.get_json_from_index(self.device,
                                                    self.channel,
                                                    0,
                                                    'http://server.com')
        # then
        self.assertThat(json_dict['version'], Equals(20130808))
        self.assertThat(json_dict['description'], Equals('20130807'))
        self.assertThat(json_dict['type'], Equals('full'))
        self.assertThat(json_dict['files'], HasLength(3))

    def testGetPreviousIndex(self, get_content_mock):
        # given
        get_content_mock.return_value = self.json_content
        # when
        json_dict = ubuntuimage.get_json_from_index(self.device,
                                                    self.channel,
                                                    -1,
                                                    'http://server.com')
        # then
        self.assertThat(json_dict['version'], Equals(20130807))
        self.assertThat(json_dict['description'], Equals('20130806.1'))
        self.assertThat(json_dict['type'], Equals('full'))
        self.assertThat(json_dict['files'], HasLength(3))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright (C) 2013 Canonical Ltd.
# Author: Sergio Schvezov <sergio.schvezov@canonical.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

import hashlib
import logging
import os.path

from phabletutils.device import AndroidBridge
from phabletutils import cdimage
from phabletutils import community
from phabletutils import downloads
from phabletutils import hashes
from phabletutils import resources
from phabletutils import projects
from phabletutils import settings
from phabletutils import ubuntuimage
from subprocess import CalledProcessError

log = logging.getLogger()


def device_friendly_error_message(f):
    def wrapper(*args, **kwargs):
        try:
            return f(*args, **kwargs)
        except CalledProcessError:
            raise EnvironmentError(
                'Device either not connected, doesn\'t have adb enabled '
                'or the property system cannot be accessed. Make sure the '
                'device is booted into the operating system and that adb '
                'is working correctly.')
    return wrapper


@device_friendly_error_message
def detect_device(serial, device=None):
    '''If no argument passed determine them from the connected device.'''
    # Check CyanogenMod property
    if not device:
        adb = AndroidBridge(serial)
        adb.start()
        device = adb.getprop('ro.cm.device').strip()
    # Check Android property
    if not device:
        device = adb.getprop('ro.product.device').strip()
    log.info('Device detected as %s' % device)
    # property may not exist or we may not map it yet
    if device not in settings.supported_devices:
        raise EnvironmentError('Unsupported device, autodetect fails device')
    return device


def setup_cdimage_files(project_name, uri, download_dir, series,
                        device, legacy=False):
    downloads.setup_download_directory(download_dir)
    templ_arch_any = settings.files_arch_any[project_name]
    templ_arch_all = settings.files_arch_all[project_name]
    file_names = {}
    for key in templ_arch_any:
        file_names[key] = templ_arch_any[key] % (series, device)
    for key in templ_arch_all:
        file_names[key] = templ_arch_all[key] % series
    if legacy:
        hash_func = hashlib.md5
        hash_dict = {}
        for key in file_names:
            file_hash = hashes.load_hash(uri, '%s.md5sum' %
                                         file_names[key], download_dir)
            hash_dict[file_names[key]] = file_hash[file_names[key]]
    else:
        hash_func = hashlib.sha256
        hash_dict = hashes.load_hash(uri, 'SHA256SUMS', download_dir)
    files = {}
    for key in file_names:
        if uri:
            file_uri = '%s/%s' % (uri, file_names[key])
        else:
            file_uri = None
        files[key] = resources.File(
            file_path=os.path.join(download_dir, file_names[key]),
            file_uri=file_uri,
            file_hash=hash_dict[file_names[key]],
            file_hash_func=hash_func)
    return files


def setup_cdimage_touch(args):
    device = detect_device(args.serial, args.device)
    series = args.series
    base_uri = '%s/%s' % (settings.cdimage_uri_base, args.project)

    if args.base_path:
        uri = None
        download_dir = args.base_path
    else:
        daily_uri = '%s/daily-preinstalled' % (base_uri, )
        build = cdimage.get_build(daily_uri, args.pending)
        uri = '%s/%s' % (daily_uri, build)
        download_dir = downloads.get_full_path(
            os.path.join(settings.download_dir, args.project, build))
    files = setup_cdimage_files(
        args.project, uri, download_dir, series, device)
    return cdimage_project(files, args)


def setup_cdimage_legacy(args):
    series = args.series
    uri = args.uri
    device = detect_device(args.serial, args.device)
    if args.base_path:
        download_dir = args.base_path
    elif args.revision or args.latest_revision:
        build = args.build
        uri = args.uri
        download_dir = downloads.get_full_path(
            os.path.join(args.project, series, build))
    else:
        base_uri = '%s/%s' % (settings.cdimage_uri_base, args.project)
        daily_uri = '%s/daily-preinstalled' % (base_uri, )
        build = cdimage.get_build(daily_uri)
        uri = '%s/%s' % (daily_uri, build)
        download_dir = downloads.get_full_path(
            os.path.join(settings.download_dir, args.project, build))
    files = setup_cdimage_files(
        args.project, uri, download_dir, series, device, legacy=True)
    return cdimage_project(files, args)


def cdimage_project(files, args):
    if args.bootstrap:
        return projects.UbuntuTouchBootstrap(
            system=files['system_img'],
            boot=files['boot_img'],
            recovery=files['recovery_img'],
            ubuntu=files['ubuntu_zip'])
    else:
        if args.device_path:
            files['device_zip'] = args.device_path
        if args.ubuntu_path:
            files['ubuntu_zip'] = args.ubuntu_path
        return projects.UbuntuTouchRecovery(
            device=files['device_zip'],
            ubuntu=files['ubuntu_zip'],
            wipe=args.wipe)


def setup_ubuntu_system(args):
    device = detect_device(args.serial, args.device)
    uri = args.alternate_server
    if args.revision <= 0:
        json = ubuntuimage.get_json_from_index(device,
                                               args.channel,
                                               args.revision,
                                               uri)
    else:
        json = ubuntuimage.get_json_from_version(device,
                                                 args.channel,
                                                 args.revision,
                                                 uri)
    download_dir_part = os.path.join(settings.download_dir, args.project)
    if uri != settings.system_image_uri:
        base_uri = downloads.get_sha256_hash(uri)
        download_dir_part = os.path.join(download_dir_part, base_uri)
    download_dir = downloads.get_full_path(download_dir_part)

    log.info('Flashing revision %s from channel %s' %
             (json['version'], args.channel))
    files, command_part = ubuntuimage.get_files(download_dir, uri, json)
    return projects.UbuntuTouchSystem(
        file_list=files,
        command_part=command_part,
        device=device,
        wipe=args.bootstrap,
        system_image_ready=args.system_image_ready)


def setup_community(args):
    config_dir = community.branch_project(args.device)
    json_dict = community.load_manifest(config_dir)
    download_dir = community.get_download_dir(
        args.device, json_dict['revision'])
    files = community.get_files(json_dict, download_dir, args.series)
    return projects.UbuntuTouchRecovery(
        device=files['device'],
        ubuntu=files['ubuntu'],
        storage=json_dict['storage'],
        wipe=args.wipe)


def list_revisions(args):
    # Easy hack to get rid of the logger and inhibit requests from logging
    log.setLevel(logging.FATAL)
    revisions = cdimage.get_available_revisions(args.uri)
    cdimage.display_revisions(revisions)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright (C) 2013 Canonical Ltd.
# Author: Sergio Schvezov <sergio.schvezov@canonical.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Resources for downloading and installing Ubuntu Touch."""

import downloads
import hashlib
import logging

log = logging.getLogger()


class File(object):
    """A file object that can be downloaded and flashed."""

    @property
    def uri(self):
        return self._uri

    @property
    def path(self):
        return self._path

    @property
    def verified(self):
        return self._verified

    @property
    def hash(self):
        return self._hash

    @property
    def hash_type(self):
        return self._hash_func

    @property
    def check(self):
        return self._check

    def __init__(self, file_uri, file_path, check=True, file_hash=None,
                 file_hash_func=hashlib.sha256):
        self._uri = file_uri
        self._path = file_path
        self._hash = file_hash
        self._hash_func = file_hash_func
        self._check = check
        if check and file_hash:
            self._verified = downloads.checksum_verify(file_path, file_hash,
                                                       file_hash_func)
            log.debug('%s verified: %s' % (file_path, self._verified))
        else:
            self._verified = False


class SignedFile(File):

    def sig_uri(self):
        return self._sig_uri

    @property
    def sig_path(self):
        return self._sig_path

    def __init__(self, file_uri, file_path, file_hash, sig_path, sig_uri,
                 check=True, file_hash_func=hashlib.sha256):
        super(SignedFile, self).__init__(
            file_uri, file_path, check, file_hash, file_hash_func)
        self._sig_path = sig_path
        self.sig_uri = sig_uri



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright (C) 2013 Canonical Ltd.
# Author: Sergio Schvezov <sergio.schvezov@canonical.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import os.path

from phabletutils import downloads

log = logging.getLogger()


def hash2dict(hash_file_content):
    '''Returns a dictionary with the sha256 sums for all files.'''
    if not hash_file_content:
        log.debug('hash file is empty')
        return None
    hash_list = filter((lambda x: len(x) is not 0),
                       hash_file_content.split('\n'))
    hash_list = [h.split() for h in hash_list]
    hash_dict = {}
    for hash_entry in hash_list:
        if hash_entry[1][0] == '*':
            hash_entry[1] = hash_entry[1][1:]
        hash_dict[hash_entry[1]] = hash_entry[0]
    return hash_dict


def load_hash(uri, artifact, download_dir=None):
    if download_dir:
        hash_path = os.path.join(download_dir, artifact)
    else:
        hash_path = None
    hashes = {}
    if hash_path and os.path.exists(hash_path):
        with open(hash_path, 'r') as f:
            file_hash_content = f.read()
        hashes = hash2dict(file_hash_content)
    if uri:
        uri = '%s/%s' % (uri, artifact)
        hash_content = downloads.get_content(uri)
        if not hash_content:
            raise RuntimeError('%s cannot be downloaded' % uri)
        hashes.update(hash2dict(hash_content))
        if hash_path:
            log.debug('Storing hash to %s' % hash_path)
            with downloads.flocked(hash_path):
                with open(hash_path, 'w') as f:
                    for key in hashes:
                        f.write('%s %s\n' % (hashes[key], key))
    if hashes:
        return hashes
    else:
        raise RuntimeError('%s cannot be obtained for verifiaction' % artifact)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright (C) 2013 Canonical Ltd.
# Author: Sergio Schvezov <sergio.schvezov@canonical.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

import json
import hashlib
import logging
import os
import os.path
import subprocess

from phabletutils import cdimage
from phabletutils import downloads
from phabletutils import resources
from phabletutils import license
from phabletutils import settings

log = logging.getLogger()
branch_template = 'lp:~{0}-image-dev/phablet-image-info/{0}'
base_dir = os.path.join(settings.download_dir, 'community')

license_template = '''This community project has the following license:

{0}
'''


def branch_project(device):
    branch = branch_template.format(device.replace('_', '-'))
    log.info('Obtaining project branch from %s' % branch)
    download_dir = downloads.get_full_path(os.path.join(base_dir, device))
    target = os.path.join(download_dir, 'config')
    if os.path.exists(target):
        subprocess.check_call(['bzr', 'update'], cwd=target)
    else:
        subprocess.check_call(
            ['bzr', 'checkout', '--lightweight', branch, target])
    log.info('Target config retrieved to %s' % target)
    # Best time to do this is right after retrieving the config
    ensure_license_accept(download_dir, os.path.join(target, 'license'))
    return target


def ensure_license_accept(download_dir, license_file):
    if not os.path.exists(license_file):
        raise EnvironmentError('Project does not offer a license file')
    accept_path = os.path.join(download_dir, '.license_accept')
    with open(license_file, 'r') as f:
        device_license = f.read()
    device_license = 'LICENSE TEXT NOT PROVIDED BY PORT MAINTAINER' \
        if not device_license else device_license
    message = license_template.format(device_license)
    if not license.has_accepted(accept_path) and \
       not license.query(message, accept_path):
        raise RuntimeError('License not accepted.')


def load_manifest(directory):
    manifest_file = os.path.join(directory, 'manifest.json')
    if not os.path.exists(manifest_file):
        raise RuntimeError('Cannot locate %s' % manifest_file)
    with open(manifest_file) as f:
        manifest_dict = json.load(f)
    if 'device' not in manifest_dict:
        raise EnvironmentError('device entry required in manifest')
    if 'revision' not in manifest_dict:
        manifest_dict['revision'] = None
    if 'storage' not in manifest_dict:
        manifest_dict['storage'] = '/sdcard/'
    if 'ubuntu' not in manifest_dict:
        manifest_dict['ubuntu'] = None
    log.debug(json.dumps(manifest_dict))
    return manifest_dict


def get_download_dir(device, revision=None):
    download_dir = os.path.join(settings.download_dir, 'community', device)
    if revision:
        download_dir = os.path.join(download_dir, revision)
    return downloads.get_full_path(download_dir)


def get_files(manifest_dict, download_dir, series):
    files = {}
    for key in ('device', 'ubuntu'):
        if isinstance(manifest_dict[key], dict):
            item = manifest_dict[key]
            hash_type = item['hash_func'] if 'hash_func' in item else None
            log.debug('%s has config uri: %s' % (key, item['uri']))
            files[key] = resources.File(
                file_path=os.path.join(download_dir, '%s.zip' % key),
                file_uri=item['uri'],
                file_hash=item['hash'] if 'hash' in item else None,
                file_hash_func=get_hash_func(hash_type) if hash_type else None)
        elif isinstance(manifest_dict[key], str) or \
                isinstance(manifest_dict[key], unicode):
            log.debug('%s has config uri: %s' % (key, manifest_dict[key]))
            files[key] = resources.File(
                file_path=os.path.join(download_dir, '%s.zip' % key),
                file_uri=manifest_dict[key],
                file_hash=None)
        log.debug('%s is type %s' % (key, type(manifest_dict[key])))
    if not manifest_dict['ubuntu']:
        log.warning('Using Ubuntu Touch current build for ubuntu image')
        files['ubuntu'] = cdimage.get_file(file_key='ubuntu_zip',
                                           series=series,
                                           download_dir=download_dir)
    return files


def get_hash_func(hash_type):
    return {
        'md5': hashlib.md5,
        'sha256': hashlib.sha256,
        }.get(hash_type, None)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright (C) 2013 Canonical Ltd.
# Author: Sergio Schvezov <sergio.schvezov@canonical.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import logging

log = logging.getLogger()


def accepted(pathname):
    '''
    Remember that the user accepted the license.
    '''
    open(pathname, 'w').close()


def has_accepted(pathname):
    '''
    Return True if the user accepted the license once.
    '''
    return os.path.exists(pathname)


def query(message, accept_path):
    '''Display end user agreement to continue with deployment.'''
    try:
        while True:
            print message
            print 'Do you accept? [yes|no]'
            answer = raw_input().lower()
            if answer == 'yes':
                accepted(accept_path)
                return True
            elif answer == 'no':
                return False
    except KeyboardInterrupt:
        log.error('Interruption detected, cancelling install')
        return False



# Copyright (C) 2013 Canonical Ltd.
# Author: Sergio Schvezov <sergio.schvezov@canonical.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Unit tests for phabletutils.environment."""

from mock import patch
from phabletutils import cdimage
from testtools import TestCase
from testtools.matchers import Equals


@patch('subprocess.check_output')
class TestCdimageLinks(TestCase):
    def setUp(self):
        super(TestCdimageLinks, self).setUp()
        self.cdimage_uri = 'http://cdimage.ubuntu.com/ubuntu-touch/daily-build'
        self.rsync_uri_part = 'rsync://cdimage.ubuntu.com/cdimage/' \
                              'ubuntu-touch/daily-build'
        self.rsync_call = ['rsync', '-l']

    def testCurrent(self, process_mock):
        """Given a cdimage uri return the linked build for current."""
        # given
        target_build = '20130714'
        process_mock.return_value = target_build
        self.rsync_call.append('%s/current' % self.rsync_uri_part)
        # when
        build = cdimage.get_build(self.cdimage_uri)
        # then
        self.assertThat(build, Equals(target_build))
        process_mock.assert_called_once_with(self.rsync_call)

    def testPending(self, process_mock):
        """Given a cdimage uri return the linked build for current."""
        # given
        target_build = '20130714.1'
        process_mock.return_value = target_build
        self.rsync_call.append('%s/pending' % self.rsync_uri_part)
        # when
        build = cdimage.get_build(self.cdimage_uri, pending=True)
        # then
        self.assertThat(build, Equals(target_build))
        process_mock.assert_called_once_with(self.rsync_call)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright (C) 2013 Canonical Ltd.
# Author: Sergio Schvezov <sergio.schvezov@canonical.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import tempfile

log = logging.getLogger()


def backup(adb, backup_file=tempfile.mkstemp()[1]):
    adb.shell('tar -czpf /tmp/backup.tar.gz '
              '/home/phablet '
              '/etc/NetworkManager/system-connections')
    log.info('Saving backup to %s' % backup_file)
    adb.pull('/tmp/backup.tar.gz', backup_file)
    return backup_file


def restore(adb, backup_file):
    adb.push(backup_file, '/tmp/backup.tar.gz')
    log.info('Restoring from %s' % backup_file)
    adb.shell('tar -xzpf /tmp/backup.tar.gz -C /')
    adb.reboot()



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright (C) 2013 Canonical Ltd.
# Author: Loïc Minier <loic.minier@ubuntu.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Static checkers against source files."""

import subprocess
import testtools

from setup import PYTHON_SCRIPTS, SH_SCRIPTS


# external script which is only pyflakes-clean, not pep8-clean
PEP8_EXCLUDES = ["repo", ]


def has_command(command):
    """Returns whether a command is in  the PATH or not."""
    returncode = subprocess.call(['which', command],
                                 stdout=open('/dev/null', 'w'))
    return returncode == 0


def get_output(command):
        proc = subprocess.Popen(command,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE)
        return proc.communicate()


class TestStaticCheckers(testtools.TestCase):
    @testtools.skipUnless(has_command("checkbashisms"),
                          "checkbashisms not found")
    def test_bashisms(self):
        (stdout, stderr) = get_output(['checkbashisms', ] + SH_SCRIPTS)
        self.assertEquals('', stdout)
        self.assertEquals('', stderr)

    @testtools.skipUnless(has_command("pep8"), "pep8 not found")
    def test_pep8(self):
        scripts = [p for p in PYTHON_SCRIPTS if p not in PEP8_EXCLUDES]
        (stdout, stderr) = get_output(['pep8', '.'] + scripts)
        self.assertEquals('', stdout)
        self.assertEquals('', stderr)

    @testtools.skipUnless(has_command("pyflakes"), "pyflakes not found")
    def test_pyflakes(self):
        (stdout, stderr) = get_output(['pyflakes', '.'] + PYTHON_SCRIPTS)
        self.assertEquals('', stdout)
        self.assertEquals('', stderr)

    def test_sh_syntax(self):
        (stdout, stderr) = get_output(['sh', '-n'] + SH_SCRIPTS)
        self.assertEquals('', stdout)
        self.assertEquals('', stderr)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright (C) 2013 Canonical Ltd.
# Author: Sergio Schvezov <sergio.schvezov@canonical.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

revision = 2
default_series = 'trusty'
cdimage_legacy_series = 'saucy'
cdimage_uri_base = 'http://cdimage.ubuntu.com'
system_image_uri = 'https://system-image.ubuntu.com'
download_dir = 'phablet-flash'

files_arch_any = {
    'ubuntu-touch': {
        'device_zip': '%s-preinstalled-touch-armel+%s.zip',
        'system_img': '%s-preinstalled-system-armel+%s.img',
        'boot_img': '%s-preinstalled-boot-armhf+%s.img',
        'recovery_img': '%s-preinstalled-recovery-armel+%s.img',
    },
    'ubuntu-touch-preview': {
        'device_zip': '%s-preinstalled-armel+%s.zip',
        'system_img': '%s-preinstalled-system-armel+%s.img',
        'boot_img': '%s-preinstalled-boot-armel+%s.img',
        'recovery_img': '%s-preinstalled-recovery-armel+%s.img',
    },
}

files_arch_all = {
    'ubuntu-touch': {
        'ubuntu_zip': '%s-preinstalled-touch-armhf.zip',
    },
    'ubuntu-touch-preview': {
        'ubuntu_zip': '%s-preinstalled-phablet-armhf.zip',
    }
}

recovery_script_template = '''boot-recovery
--update_package=/sdcard/{0}
--user_data_update_package=/sdcard/{1}
reboot
'''

supported_devices = ('mako',
                     'maguro',
                     'manta',
                     'flo',
                     'grouper',
                     'krillin',
                     )
legal_notice = '''"Touch Developer Preview for Ubuntu" is released for free
non-commercial use. It is provided without warranty, even the implied
warranty of merchantability, satisfaction or fitness for a particular
use. See the licence included with each program for details.

Some licences may grant additional rights; this notice shall not limit
your rights under each program's licence. Licences for each program
are available in the usr/share/doc directory. Source code for Ubuntu
can be downloaded from archive.ubuntu.com. Ubuntu, the Ubuntu logo
and Canonical are registered trademarks of Canonical Ltd. All other
trademarks are the property of their respective owners.

"Touch Preview for Ubuntu" is released for limited use due to the
inclusion of binary hardware support files. The original components
and licenses can be found at:
https://developers.google.com/android/nexus/drivers.
'''
accept_path = '~/.phablet_accepted'



# Copyright (C) 2013 Canonical Ltd.
# Author: Sergio Schvezov <sergio.schvezov@canonical.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Unit tests for phabletutils.environment."""

import json
import shutil
import tempfile

from os import path
from phabletutils import community
from testtools import TestCase
from testtools.matchers import Equals
from testtools.matchers import Is


class TestCommunityManifestLoad(TestCase):

    def setUp(self):
        super(TestCommunityManifestLoad, self).setUp()
        self.device_dir = tempfile.mkdtemp()

    def tearDown(self):
        super(TestCommunityManifestLoad, self).tearDown()
        shutil.rmtree(self.device_dir)

    def testSimpleDeviceManifest(self):
        # given
        manifest = {'device': 'http://somelocation.com/device.zip'}
        with open(path.join(self.device_dir, 'manifest.json'), 'w') as f:
                f.write(json.dumps(manifest))
        # when
        loaded_manifest = community.load_manifest(self.device_dir)
        # then
        self.assertThat(loaded_manifest['device'], Equals(manifest['device']))
        self.assertThat(loaded_manifest['ubuntu'], Is(None))
        self.assertThat(loaded_manifest['revision'], Is(None))
        self.assertThat(loaded_manifest['storage'], Equals('/sdcard/'))

    def testSimpleDeviceAndUbuntuManifest(self):
        # given
        manifest = {'device': 'http://somelocation.com/device.zip',
                    'ubuntu': 'http://somelocation.com/ubuntu.zip'}
        with open(path.join(self.device_dir, 'manifest.json'), 'w') as f:
                f.write(json.dumps(manifest))
        # when
        loaded_manifest = community.load_manifest(self.device_dir)
        # then
        self.assertThat(loaded_manifest['device'], Equals(manifest['device']))
        self.assertThat(loaded_manifest['ubuntu'], Equals(manifest['ubuntu']))
        self.assertThat(loaded_manifest['revision'], Is(None))
        self.assertThat(loaded_manifest['storage'], Equals('/sdcard/'))

    def testSimpleDeviceAndUbuntuRevisionManifest(self):
        # given
        manifest = {'device': 'http://somelocation.com/device.zip',
                    'ubuntu': 'http://somelocation.com/ubuntu.zip',
                    'revision': 'unstable'}
        with open(path.join(self.device_dir, 'manifest.json'), 'w') as f:
                f.write(json.dumps(manifest))
        # when
        loaded_manifest = community.load_manifest(self.device_dir)
        # then
        self.assertThat(loaded_manifest['device'], Equals(manifest['device']))
        self.assertThat(loaded_manifest['ubuntu'], Equals(manifest['ubuntu']))
        self.assertThat(loaded_manifest['revision'],
                        Equals(manifest['revision']))
        self.assertThat(loaded_manifest['storage'], Equals('/sdcard/'))

    def testSimpleDeviceAndUbuntuRevisionStorageManifest(self):
        # given
        manifest = {'device': 'http://somelocation.com/device.zip',
                    'ubuntu': 'http://somelocation.com/ubuntu.zip',
                    'storage': '/emmc/',
                    'revision': 'unstable'}
        with open(path.join(self.device_dir, 'manifest.json'), 'w') as f:
                f.write(json.dumps(manifest))
        # when
        loaded_manifest = community.load_manifest(self.device_dir)
        # then
        self.assertThat(loaded_manifest['device'], Equals(manifest['device']))
        self.assertThat(loaded_manifest['ubuntu'], Equals(manifest['ubuntu']))
        self.assertThat(loaded_manifest['revision'],
                        Equals(manifest['revision']))
        self.assertThat(loaded_manifest['storage'],
                        Equals(manifest['storage']))


class TestCommunityFiles(TestCase):

    def setUp(self):
        super(TestCommunityFiles, self).setUp()
        self.download_dir = '/downloads'
        self.series = 'saucy'

    def tearDown(self):
        super(TestCommunityFiles, self).tearDown()

    def testSimpleDeviceAndUbuntu(self):
        # given
        manifest = {'device': 'http://somelocation.com/device.zip',
                    'ubuntu': 'http://somelocation.com/ubuntu.zip'}
        # when
        files = community.get_files(manifest, self.download_dir, self.series)
        # then
        self.assertThat(files['device'].uri, Equals(manifest['device']))
        self.assertThat(files['device'].hash, Is(None))
        self.assertThat(files['device'].verified, Is(False))
        self.assertThat(files['device'].path,
                        Equals(path.join(self.download_dir,
                               path.basename(manifest['device']))))
        self.assertThat(files['ubuntu'].uri, Equals(manifest['ubuntu']))
        self.assertThat(files['ubuntu'].hash, Is(None))
        self.assertThat(files['ubuntu'].verified, Is(False))
        self.assertThat(files['ubuntu'].path,
                        Equals(path.join(self.download_dir,
                               path.basename(manifest['ubuntu']))))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright (C) 2013 Canonical Ltd.
# Author: Sergio Schvezov <sergio.schvezov@canonical.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import re
import requests
import os.path
import subprocess
import urlparse

from phabletutils import hashes
from phabletutils import fileutils
from phabletutils import resources
from phabletutils import settings

log = logging.getLogger()


def _get_elements(uri):
    '''Scraps cdimage and returns a list of relevant links as elements.'''
    request = requests.get(uri).content
    html_elements = filter(
        lambda x: '<li><a href=' in x and
        'Parent Directory' not in x and
        'daily' not in x and
        'current' not in x,
        request.split('\n'))
    # cdimage lays out the content in more than one way.
    if not html_elements:
        html_elements = filter(
            lambda x: 'alt="[DIR]"' in x and
            'Parent Directory' not in x and
            'daily' not in x and
            'pending' not in x and
            'current' not in x,
            request.split('\n'))
        html_elements = [re.search(r'<a.* href.*>.*</a>', x).group(0)
                         for x in html_elements]
    elements_inter = [re.sub('<[^>]*>', '', i) for i in html_elements]
    elements = [i.strip().strip('/') for i in elements_inter]
    return elements


def _get_releases(uri):
    '''Fetches the available releases for a given cdimage project URI.'''
    releases = [{'release': i, 'uri': '%s/%s' % (uri, i)}
                for i in _get_elements(uri)]
    log.debug('releases: %s', releases)
    return releases


def _get_revisions(uri):
    '''Returns the revisions for a given release URI.'''
    return _get_elements(uri)


def get_available_revisions(cdimage_uri):
    '''Returns all the releases available for a given cdimage project.'''
    if cdimage_uri[-1] != '/':
        cdimage_uri = cdimage_uri + '/'
    releases = _get_releases(cdimage_uri)
    for release in releases:
        release['revisions'] = _get_revisions(release['uri'])
    return releases


def get_latest_revision(cdimage_uri):
    '''Returns the latest revision tagged.'''
    log.debug('cdimage_uri: %s', cdimage_uri)
    try:
        latest_release = _get_releases(cdimage_uri)[-1]
        log.debug('latest_release: %s', latest_release)
        latest_revision = _get_revisions(latest_release['uri'])[-1]
    except IndexError:
        raise EnvironmentError('No releases for current project. Verify '
                               'by checking %s' % cdimage_uri)
    return latest_release['release'], latest_revision


def display_revisions(revisions):
    '''Displays the available revisions for a give cdimage project.'''
    for series in revisions:
        print 'Available releases:'
        if 'revisions' in series:
            for rev in series['revisions']:
                print '\t%s/%s' % (series['release'], rev)
        else:
            print 'No releases for %s available' % rev['release']
    if not revisions:
        print 'No revisions have been tagged for this project yet'


def _get_link_target(uri):
    '''Returns the target for the link in the image.'''
    uri = urlparse.urlparse(uri)
    uri = 'rsync://%s/cdimage%s' % (uri.netloc, uri.path)
    link = subprocess.check_output(['rsync', '-l', uri]).split()[-1]
    return link


def get_build(cdimage_uri, pending=False):
    '''Returns the latest build in current.'''
    if pending:
        uri = '%s/%s' % (cdimage_uri, 'pending')
    else:
        uri = '%s/%s' % (cdimage_uri, 'current')
    build = _get_link_target(uri)
    return build


def get_file(file_key, series, download_dir, project='ubuntu-touch',
             device=None):
    uri = '%s/%s/daily-preinstalled' % (settings.cdimage_uri_base, project)
    build = get_build(uri)
    hash_dict = hashes.load_hash('%s/%s' % (uri, build), 'SHA256SUMS')
    if device:
        file_name = settings.files_arch_any[project][file_key] %\
            (series, device)
        download_dir = fileutils.create_path(
            download_dir, os.path.join(device, build))
    else:
        file_name = settings.files_arch_all[project][file_key] % series
        download_dir = fileutils.create_path(
            download_dir, build)
    log.debug('Download dir for %s set to %s' % (file_name, download_dir))
    return resources.File(
        file_path=os.path.join(download_dir, file_name),
        file_uri='%s/%s/%s' % (uri, build, file_name),
        file_hash=hash_dict[file_name])



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright (C) 2013 Canonical Ltd.
# Author: Sergio Schvezov <sergio.schvezov@canonical.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import atexit
import contextlib
import logging
import lzma
import os.path
import shutil
import tarfile
import tempfile


log = logging.getLogger()


def cleanup(directory):
    if os.path.exists(directory) and os.path.isdir(directory):
        print('Removing directory %s' % directory)
        shutil.rmtree(directory)


def create_temp_dir():
    directory = tempfile.mkdtemp()
    atexit.register(cleanup, directory)
    return directory


def create_path(directory, subdir):
    directory = os.path.join(directory, subdir)
    if not os.path.exists(directory):
        log.debug('Creating directory %s' % directory)
        try:
            os.makedirs(directory)
        except OSError as e:
            if e.errno == 17:
                pass
            else:
                raise e
    return directory


def lzma_extract_file(lzma_tarfile, file_name):
    log.info('Decompressing %s from %s' % (file_name, lzma_tarfile))
    temp_dir = create_temp_dir()
    with contextlib.closing(lzma.LZMAFile(lzma_tarfile)) as xz:
        with tarfile.open(fileobj=xz) as tar:
            tar.extract(member=file_name, path=temp_dir)
    return os.path.join(temp_dir, file_name)



#!/usr/bin/python

from setuptools import find_packages
from setuptools import setup

PYTHON_SCRIPTS = [
    'phablet-click-test-setup',
    'phablet-demo-setup',
    'phablet-dev-bootstrap',
    'phablet-flash',
    'repo',
    'phablet-config'
    ]

SH_SCRIPTS = [
    'citrain',
    'phablet-network',
    'phablet-screenshot',
    'phablet-test-run',
    'click-buddy',
    'phablet-bootchart',
    'phablet-shell',
    ]

DATA_FILES = [
    ('/usr/share/phabletutils/', ['shell-adb-common.sh', ]),
    ]

if __name__ == "__main__":
    setup(
        name='phablet-tools',
        version='0.1',
        description='Scripts to deploy Ubuntu on mobile devices',
        author='Sergio Schvezov',
        author_email='sergio.schvezov@canonical.com',
        license='GPLv3',
        packages=find_packages(exclude=("tests",)),
        scripts=PYTHON_SCRIPTS + SH_SCRIPTS,
        data_files=DATA_FILES,
        test_suite='tests',
    )



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright (C) 2013 Canonical Ltd.
# Author: Sergio Schvezov <sergio.schvezov@canonical.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Holds different projects or ways Ubuntu Touch is delivered."""

import os
import os.path
import tempfile
import logging
import subprocess
import gzip

from phabletutils.downloads import checksum_verify
from phabletutils.resources import (File, SignedFile)
from phabletutils import downloads
from phabletutils import fileutils
from time import sleep
from textwrap import dedent

log = logging.getLogger()


def gunzip(file_path):
    if not file_path.endswith('.gz'):
        return file_path
    target_path = file_path[:-3]
    log.info('Decompressing %s' % file_path)
    with open(target_path, 'wb') as target_file:
        with gzip.open(file_path, 'r') as gzip_file:
            for chunk in gzip_file.read():
                target_file.write(chunk)
    return target_path


def wipe_device(adb):
    log.info('Clearing /data and /cache')
    adb.shell('mount /data')
    adb.shell('rm -Rf /cache/* /data/* /data/.developer_mode '
              '/data/.writable_image')
    adb.shell('mkdir /cache/recovery')
    adb.shell('mkdir /data/media')


class BaseProject(object):
    """A provisioning mechanism for all the projects."""

    def __init__(self, recovery=None, system=None, boot=None,
                 device=None, ubuntu=None, wipe=False):
        self._list = []
        for item in (recovery, system, boot, device, ubuntu):
            if item and not isinstance(item, File):
                raise TypeError('%s is not of type File' % item)
            elif item:
                self._list.append(item)
                if item.check and not item.uri and not item.verified:
                    raise EnvironmentError(
                        '%s is not on disk' % item.path)

        self._recovery = recovery
        self._system = system
        self._boot = boot
        self._device = device
        self._ubuntu = ubuntu
        self._wipe = wipe

    def download(self):
        """Downloads and verifies resources."""
        download_list = filter((lambda x: x.check), self._list)
        download_list = filter((lambda x: not x.verified), download_list)
        log.debug('Download list %s' % download_list)
        if not download_list:
            log.info('Download not required')
            return
        for entry in download_list:
            log.debug('Download entry %s %s' % (entry.path, entry.verified))
            downloads.download(entry)
            if entry.hash and \
               not checksum_verify(entry.path, entry.hash, entry.hash_type):
                raise EnvironmentError(
                    'Checksum does not match after download for %s '
                    'and hash %s' % (entry.path, entry.hash))
        download_list = filter(lambda x: isinstance(x, SignedFile), self._list)
        for entry in download_list:
            downloads.download_sig(entry)

    def install(self):
        raise EnvironmentError('Requires implementation')

    def _loop_until_installed(self, adb):
        log.info('Waiting for install to finish on device. Please do not '
                 'unplug device until phablet-flash finishes.')
        # Initial wait time
        sleep(180)
        wait_try = 100
        install_success = False
        for wait in range(wait_try):
            sleep(10)
            try:
                output = adb.shell('pgrep unity8 2>/dev/null')
            except:
                output = None
            if output:
                install_success = True
                break
            wait += 1
        if not install_success:
            raise RuntimeError('Installation is taking too long or an error '
                               'occured along the way.')
        log.info('Installation complete')


class Android(BaseProject):
    """Standard Android Project."""

    def __init__(self, boot, system):
        super(Android, self).__init__(boot=boot, system=system)

    def install(self, adb, fastboot):
        adb.reboot(bootloader=True)
        log.warning('Device needs to be unlocked for the following to work')
        fastboot.flash('system', gunzip(self._system.path))
        fastboot.flash('boot', self._boot.path)
        log.info('Installation will complete soon and reboot into Android')
        fastboot.reboot()


class UbuntuTouchBootstrap(BaseProject):

    def __init__(self, boot, system, recovery, ubuntu):
        log.debug('UbuntuTouchBootstrap '
                  'boot: %s, system: %s, recovery: %s, ubuntu: %s' %
                  (boot.path, system.path, recovery.path, ubuntu.path))
        super(UbuntuTouchBootstrap, self).__init__(
            boot=boot, system=system, recovery=recovery, ubuntu=ubuntu,
            wipe=True)

    def install(self, adb, fastboot):
        adb.reboot(bootloader=True)
        log.warning('Device needs to be unlocked for the following to work')
        fastboot.flash('system', self._system.path)
        fastboot.flash('boot', self._boot.path)
        fastboot.flash('recovery', self._recovery.path)
        fastboot.boot(self._recovery.path)
        adb.wait_for_recovery()
        wipe_device(adb)
        adb.push(self._ubuntu.path, '/sdcard/autodeploy.zip')
        log.info('Deploying Ubuntu')
        adb.reboot(recovery=True)
        log.info('Once completed the device should reboot into Ubuntu')


class UbuntuTouchRecovery(BaseProject):

    recovery_script_template = dedent('''\
                                      mount("{0}");
                                      install_zip("{1}");
                                      install_zip("{2}");
                                      ''')

    def __init__(self, device, ubuntu, storage='/sdcard/', wipe=False):
        log.debug('UbuntuTouchRecovery device: %s, ubuntu: %s, wipe: %s' %
                  (device.path, ubuntu.path, wipe))
        super(UbuntuTouchRecovery, self).__init__(
            ubuntu=ubuntu, device=device, wipe=wipe)
        self._storage = storage

    def install(self, adb, fastboot=None):
        """
        Deploys recovery files, recovery script and then reboots to install.
        """
        log.warning('The device needs to have a clockwork mod recovery image '
                    '(or one that supports extendedcommands) '
                    'in place for the provisioning to work')
        adb.reboot(recovery=True)
        adb.wait_for_recovery()
        if self._wipe:
            wipe_device(adb)
        adb.shell('mount %s' % self._storage)
        adb.push(self._device.path, self._storage)
        adb.push(self._ubuntu.path, self._storage)
        recovery_file = self.create_recovery_file()
        adb.push(recovery_file, '/cache/recovery/extendedcommand')
        adb.reboot(recovery=True)
        log.info('Once completed the device should reboot into Ubuntu')
        log.debug('Removing recovery file %s' % recovery_file)
        os.unlink(recovery_file)

    def create_recovery_file(self):
        template = self.recovery_script_template
        recovery_file = tempfile.NamedTemporaryFile(delete=False)
        device = os.path.join(self._storage,
                              os.path.basename(self._device.path))
        ubuntu = os.path.join(self._storage,
                              os.path.basename(self._ubuntu.path))
        recovery_script = template.format(self._storage, device, ubuntu)
        with recovery_file as output_file:
            output_file.write(recovery_script)
        return recovery_file.name


class UbuntuTouchSystem(BaseProject):

    ubuntu_recovery_script = dedent('''\
        format system
        load_keyring image-master.tar.xz image-master.tar.xz.asc
        load_keyring image-signing.tar.xz image-signing.tar.xz.asc
        mount system
        ''')

    def __init__(self, file_list, device, command_part, wipe,
                 system_image_ready):
        log.debug('UbuntuTouchSystem')
        super(UbuntuTouchSystem, self).__init__(wipe=wipe)
        for item in file_list:
            if item and not isinstance(item, File):
                raise TypeError('%s is not of type File' % item)
            elif item:
                self._list.append(item)
        self._recovery_list = file_list
        self._command_part = command_part
        self._device = device
        self._system_image_ready = system_image_ready

    def _is_device_file(self, entry):
        f = os.path.basename(entry.path)
        return (f.startswith('device') or f.startswith(self._device)) \
            and f.endswith('.xz')

    def _get_recovery_image(self):
        recovery_path = None
        for entry in self._recovery_list:
            if self._is_device_file(entry):
                recovery_path = fileutils.lzma_extract_file(
                    entry.path, 'partitions/recovery.img')
        if not recovery_path:
            raise RuntimeError('Cannot find recovery image to flash '
                               'from downloaded files')
        log.debug('Recovery found in %s' % recovery_path)
        return recovery_path

    def _get_current_version(self, adb):
        try:
            image_info = adb.shell('system-image-cli -i',
                                   ignore_errors=False).split('\r\n')
            version = [i.split(':')[1].strip() for i in image_info
                       if 'build number: ' in i][0]
            log.debug('Current version on system is %s' % version)
            return version
        except subprocess.CalledProcessError:
            return '0'

    def install(self, adb, fastboot=None):
        """
        Deploys recovery files, recovery script and then reboots to install.
        """
        if self._system_image_ready:
            log.warning('Not doing any verification that the system '
                        'has the required dependencies before flashing')
        elif not self._wipe and self._get_current_version(adb) == '0':
            raise RuntimeError(
                'Backup requested but cannot be completed succesfully, try '
                'with --system-image-ready if the system is already on an '
                'Image Based Ubuntu System to force it or use --bootstrap '
                'if data saving is not important or the system is not already '
                'on an Image Based Ubuntu System.')

        recovery_path = self._get_recovery_image()
        adb.reboot(bootloader=True)
        fastboot.boot(recovery_path)
        adb.wait_for_recovery()
        if self._wipe:
            wipe_device(adb)

        for entry in self._recovery_list:
            adb.push(entry.path, '/cache/recovery/')
            try:
                adb.push(entry.sig_path, '/cache/recovery/')
            except AttributeError:
                pass

        adb.push(self.create_ubuntu_command_file(),
                 '/cache/recovery/ubuntu_command')
        adb.reboot(bootloader=True)
        fastboot.flash('recovery', recovery_path)
        fastboot.boot(recovery_path)
        self._loop_until_installed(adb)

    def create_ubuntu_command_file(self):
        ubuntu_command_file = tempfile.NamedTemporaryFile(delete=False)
        with ubuntu_command_file as output_file:
            if self._wipe:
                output_file.write('format data\n')
            output_file.write(self.ubuntu_recovery_script)
            output_file.write(self._command_part)
            output_file.write('unmount system\n')
        return ubuntu_command_file.name
